@@ -9,7 +9,6 @@ import pymc as pm
 import numpy as np
 import pandas as pd
 import nose
-pd.set_printoptions(precision=4)
 from nose import SkipTest
 
 import hddm
@@ -52,7 +51,7 @@ class TestSingleBreakdown(unittest.TestCase):
         self.model_classes = [hddm.models.HDDMTruncated, hddm.models.HDDMTransformed,
                               hddm.models.HDDM]
 
-        self.iter = 40
+        self.iter = 200
         self.burn = 10
 
     def runTest(self):
@@ -194,7 +193,7 @@ class TestHDDMRegressor(unittest.TestCase):
     def __init__(self, *args, **kwargs):
         super(TestHDDMRegressor, self).__init__(*args, **kwargs)
 
-        self.iter = 40
+        self.iter = 200
         self.burn = 10
 
     def runTest(self):
@@ -209,7 +208,7 @@ class TestHDDMRegressor(unittest.TestCase):
         m.sample(self.iter, burn=self.burn)
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0], pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1].__name__, 'v_cov_subj.0')
         self.assertEqual(len(np.unique(m.nodes_db.ix['wfpt.0']['node'].parents['v'].value)), 1)
@@ -225,10 +224,10 @@ class TestHDDMRegressor(unittest.TestCase):
         m.sample(self.iter, burn=self.burn)
 
         self.assertIn('z', m.include)
-        self.assertIn('z', m.nodes_db.knode_name)
+        self.assertIn('z_Intercept', m.nodes_db.knode_name)
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['z'].parents['args'][0].parents['ltheta'],
                                    pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['z'].parents['args'][0].__name__, 'z_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['z'].parents['args'][0].__name__, 'z_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['z'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['z'].parents['args'][1].__name__, 'z_cov_subj.0')
         self.assertEqual(len(np.unique(m.nodes_db.ix['wfpt.0']['node'].parents['z'].value)), 1)
@@ -241,10 +240,9 @@ class TestHDDMRegressor(unittest.TestCase):
         del data['subj_idx']
         m = hddm.HDDMRegressor(data, 'v ~ cov', group_only_regressors=False)
         m.sample(self.iter, burn=self.burn)
-        print m.nodes_db.index
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt']['node'].parents['v'].parents['args'][0], pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt']['node'].parents['v'].parents['args'][0].__name__, 'v')
+        self.assertEqual(m.nodes_db.ix['wfpt']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt']['node'].parents['v'].parents['args'][1].__name__, 'v_cov')
         self.assertEqual(len(np.unique(m.nodes_db.ix['wfpt']['node'].parents['v'].value)), 1)
@@ -259,7 +257,7 @@ class TestHDDMRegressor(unittest.TestCase):
         m.sample(self.iter, burn=self.burn)
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0], pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1].__name__, 'v_cov1_subj.0')
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][2].__name__, 'v_cov2_subj.0')
@@ -276,8 +274,8 @@ class TestHDDMRegressor(unittest.TestCase):
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0], pm.Normal))
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['a'].parents['args'][0], pm.Gamma))
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_subj.0')
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['a'].parents['args'][0].__name__, 'a_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['a'].parents['args'][0].__name__, 'a_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['a'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1].__name__, 'v_cov1_subj.0')
@@ -292,7 +290,7 @@ class TestHDDMRegressor(unittest.TestCase):
         m.sample(self.iter, burn=self.burn)
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0], pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt.0']['node'].parents['v'].parents['args'][1].__name__, 'v_cov')
         self.assertEqual(len(np.unique(m.nodes_db.ix['wfpt.0']['node'].parents['v'].value)), 1)
@@ -304,7 +302,7 @@ class TestHDDMRegressor(unittest.TestCase):
         data['cov'] = 1.
         # Create one merged column
         data['condition2'] = 'merged'
-        data[data.condition == 'c1']['condition2'] = 'single'
+        data.ix[data.condition == 'c1', 'condition2'] = 'single'
         self.assertRaises(AssertionError, hddm.HDDMRegressor, data, 'v ~ cov', depends_on={'v_Intercept': 'condition2'}, group_only_regressors=True)
 
     def test_contrast_coding(self):
@@ -318,22 +316,91 @@ class TestHDDMRegressor(unittest.TestCase):
         m.sample(self.iter, burn=self.burn)
 
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].parents['args'][0], pm.Normal))
-        self.assertEqual(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].parents['args'][0].__name__, 'v_subj.0')
+        self.assertEqual(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].parents['args'][0].__name__, 'v_Intercept_subj.0')
         self.assertTrue(isinstance(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].parents['args'][1], pm.Normal))
         self.assertEqual(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].parents['args'][1].__name__, 'v_C(condition)[T.c1]_subj.0')
         self.assertEqual(len(np.unique(m.nodes_db.ix['wfpt(c1).0']['node'].parents['v'].value)), 1)
 
+    def test_categorical_wo_intercept(self):
+        params = hddm.generate.gen_rand_params(cond_dict={'a': [1, 2, 3]})
+        data, params_true = hddm.generate.gen_rand_data(params[0], size=10, subjs=4)
+        data = pd.DataFrame(data)
+        data['cov'] = 1.
+        m = hddm.HDDMRegressor(data, 'a ~ 0 + C(condition) * cov',
+                               group_only_regressors=False)
+        m.sample(self.iter, burn=self.burn)
+
+        self.assertIsInstance(m.nodes_db.ix['a_C(condition)[c0]_subj.0']['node'], pm.Gamma)
+        self.assertIsInstance(m.nodes_db.ix['a_C(condition)[c1]_subj.0']['node'], pm.Gamma)
+        self.assertIsInstance(m.nodes_db.ix['a_C(condition)[c2]_subj.0']['node'], pm.Gamma)
+        self.assertNotIsInstance(m.nodes_db.ix['a_C(condition)[T.c1]:cov_subj.0']['node'], pm.Gamma)
+        self.assertNotIsInstance(m.nodes_db.ix['a_C(condition)[T.c2]:cov_subj.0']['node'], pm.Gamma)
+        self.assertNotIsInstance(m.nodes_db.ix['a_cov_subj.0']['node'], pm.Gamma)
 
 def test_posterior_plots_breakdown():
     params = hddm.generate.gen_rand_params()
     data, params_subj = hddm.generate.gen_rand_data(params=params, subjs=4)
     m = hddm.HDDM(data)
-    m.sample(100, burn=10)
+    m.sample(2000, burn=10)
     m.plot_posterior_predictive()
     m.plot_posterior_quantiles()
     m.plot_posteriors()
     ppc = hddm.utils.post_pred_gen(m, samples=10)
     hddm.utils.post_pred_stats(data, ppc)
 
+
+class TestRecovery(unittest.TestCase):
+    def __init__(self, *args, **kwargs):
+        super(TestRecovery, self).__init__(*args, **kwargs)
+        self.iter = 2000
+        self.burn = 20
+        np.random.seed(1)
+
+    def runTest(self):
+        return
+
+    @SkipTest
+    def test_HLBA_flat(self):
+        params_true = {'A': .5, 'b': 1, 't': .3, 's': .5, 'v': [.5, .6, .7, .8]}
+        extended_params, merged_params = extend_params(params_true)
+
+        data, params_true = hddm.models.hlba_truncated.gen_rand_data(extended_params, size=100, subjs=1)
+        model = hddm.models.HLBA(data, depends_on={'v': 'condition'})
+        model.find_starting_values()
+        model.sample(self.iter, burn=self.burn)
+        model.gen_stats()
+        model.print_stats()
+        for param, true_val in merged_params.iteritems():
+            np.testing.assert_almost_equal(true_val, model.nodes_db.ix[param]['mean'])
+
+    @SkipTest
+    def test_HLBA_hierarchical(self):
+        params = hddm.models.hlba_truncated.gen_rand_params(cond_dict={'v': [.5, .6, .75, .8]})
+        data, params_true = hddm.models.hlba_truncated.gen_rand_data(params, size=100, subjs=10)
+        model = hddm.models.HLBA(data, depends_on={'v': 'condition'})
+        model.find_starting_values()
+        model.sample(self.iter, burn=self.burn)
+
+
+def extend_params(params):
+    # Find list
+    extend_param = [param for param, val in params.iteritems() if isinstance(val, (list, tuple))]
+    if len(extend_param) > 1:
+        raise ValueError('Only one parameter can be extended')
+    extend_param = extend_param[0]
+
+    fixed_params = [param for param, val in params.iteritems() if not isinstance(val, (list, tuple))]
+
+    out_extended = {}
+    out_merged = {k: params[k] for k in fixed_params}
+    for i_cond, extend_val in enumerate(params[extend_param]):
+         cond_params = {k: params[k] for k in fixed_params}
+         cond_params[extend_param] = extend_val
+         out_extended['cond%i' % i_cond] = cond_params
+
+         out_merged['%s(cond%i)' % (extend_param, i_cond)] = extend_val
+
+    return out_extended, out_merged
+
 if __name__=='__main__':
     print "Run nosetest.py"
